/* Calendar Month Object
 * Copyright (C) 2002  Sebastian Rittau <srittau@jroger.in-berlin.de>
 *
 * $Id: calendar-month.c,v 1.1.2.3 2002/05/27 18:44:01 srittau Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include <time.h>

#include <glib.h>

#include <gnome.h>

#include "calendar-month.h"
#include "gnome-month-item.h" /* FIXME: create calendar-month-item.[ch] */
#include "gnomecal-marshal.h"
#include "mark.h" /* FIXME: move into calendar-month-item.[ch] */

static void calendar_month_init		(CalendarMonth		*cm);
static void calendar_month_finalize	(GObject		*object);
static void calendar_month_setup	(CalendarMonth		*cm);
static void calendar_month_class_init	(CalendarMonthClass	*klass);


struct _CalendarMonthPriv {
	GnomeCanvasItem *month_item;

	gint year;
	gint month;
	gint day;
};

enum {
	SIGNAL_SELECTION_CHANGED,
	SIGNAL_LAST
};


static GnomeCanvasClass *parent_class = NULL;

static guint signals[SIGNAL_LAST] = { 0 };

/*
 * Class and Instance Setup
 */

GType
calendar_month_get_type (void)
{
	static GType calendar_month_type = 0;

	if (!calendar_month_type) {
		static const GTypeInfo calendar_month_info = {
			sizeof (CalendarMonthClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) calendar_month_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (CalendarMonth),
			1,    /* n_preallocs */
			(GInstanceInitFunc) calendar_month_init,
		};

		calendar_month_type = g_type_register_static (GNOME_TYPE_CANVAS,
							      "CalendarMonth",
							      &calendar_month_info,
							      0);
	}

	return calendar_month_type;
}

static void
calendar_month_class_init (CalendarMonthClass *klass)
{
	GObjectClass *gobject_class;

	g_return_if_fail (klass != NULL);
	g_return_if_fail (CALENDAR_IS_MONTH_CLASS (klass));

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->finalize = calendar_month_finalize;

	parent_class = g_type_class_peek_parent (klass);

	/* Signal setup */

	signals[SIGNAL_SELECTION_CHANGED] =
		g_signal_new ("selection-changed",
			      G_TYPE_FROM_CLASS (klass),
			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
			      G_STRUCT_OFFSET (CalendarMonthClass, selection_changed),
			      NULL, NULL,
			      gnomecal_marshal_VOID__VOID,
			      G_TYPE_NONE, 0);
}

static void
calendar_month_init (CalendarMonth *cm)
{
	time_t t;
	struct tm tm;

	g_return_if_fail (cm != NULL);
	g_return_if_fail (CALENDAR_IS_MONTH (cm));

	cm->priv = g_new0 (CalendarMonthPriv, 1);

	t = time(NULL);
	localtime_r (&t, &tm);

	cm->priv->year  = tm.tm_year + 1900;
	cm->priv->month = tm.tm_mon  + 1;
	cm->priv->day   = tm.tm_mday;

	calendar_month_setup (cm);
}

static void
calendar_month_finalize (GObject *object)
{
	CalendarMonth *cm;

	g_return_if_fail (object != NULL);
	g_return_if_fail (CALENDAR_IS_MONTH (object));

	cm = CALENDAR_MONTH (object);

	g_free (cm->priv);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

/*
 * Internal Methods
 */

static void
calendar_month_update (CalendarMonth *cm)
{
	CalendarMonthPriv *priv;

	priv = cm->priv;

	gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->month_item),
			       "year",  priv->year,
			       "month", priv->month,
			       "day",   priv->day,
			       NULL);
}

/*
 * Callbacks
 */

/* FIXME: stolen from old goto.c; need to put that into calendar-month-item.c */
static gint
calendar_month_selection_changed (GnomeCanvasItem *item,
				  GdkEvent *event,
				  gpointer data)
{
	CalendarMonthPriv *priv;

	g_return_val_if_fail (item != NULL, FALSE);
	g_return_val_if_fail (GNOME_IS_CANVAS_ITEM (item), FALSE);
	g_return_val_if_fail (data != NULL, FALSE);
	g_return_val_if_fail (CALENDAR_IS_MONTH (data), FALSE);

	priv = CALENDAR_MONTH (data)->priv;

	if (event->type == GDK_BUTTON_PRESS && event->button.button == 1) {
		gint  child_num;
		guint day;

		child_num = gnome_month_item_child2num (GNOME_MONTH_ITEM (priv->month_item), item);
		day = gnome_month_item_num2day (GNOME_MONTH_ITEM (priv->month_item), child_num);

		if (day != 0 && day != priv->day) {
			priv->day = day;
			calendar_month_update (CALENDAR_MONTH (data));
			g_signal_emit (G_OBJECT (data),
				       signals[SIGNAL_SELECTION_CHANGED], 0);
		}
	}

	return FALSE;
}

/*
 * Setup
 */

/* FIXME: hmm... */
/* Sets the scrolling region of the canvas to the allocation size */
static void
set_scroll_region (GtkWidget *widget, GtkAllocation *allocation)
{
	CalendarMonthPriv *priv = CALENDAR_MONTH (widget)->priv;

        gnome_canvas_item_set (GNOME_CANVAS_ITEM (priv->month_item),
                               "width", (double) (allocation->width - 1),
                               "height", (double) (allocation->height - 1),
                               NULL);

        gnome_canvas_set_scroll_region (GNOME_CANVAS (widget),
                                        0, 0,
                                        allocation->width, allocation->height);
}

static void
calendar_month_setup (CalendarMonth *cm)
{
	CalendarMonthPriv *priv;

	int i;

	priv = cm->priv;

	gtk_widget_set_size_request (GTK_WIDGET (cm), 150, 120);

	/* FIXME: The following stuff is ugly heritage from pre-Gnome2
	 * gnome-pim versions. Need to clean that up by creating a new
	 * class CalendarMonthItem that will use code from GnomeMonthItem.
	 */
	priv->month_item = gnome_month_item_new (gnome_canvas_root (GNOME_CANVAS (cm)));

	unmark_month_item (GNOME_MONTH_ITEM (priv->month_item));
	calendar_month_update (cm);

	/* Connect to size_allocate so that we can change the size of the month item and the
	 * scrolling region appropriately.
	 */

	g_signal_connect (G_OBJECT (cm), "size_allocate",
			  G_CALLBACK (set_scroll_region), NULL);

	for (i = 0; i < 42; i++) {
		GnomeCanvasItem *day_group;

		day_group = gnome_month_item_num2child (GNOME_MONTH_ITEM (priv->month_item),
							i + GNOME_MONTH_ITEM_DAY_GROUP);
		g_signal_connect (G_OBJECT (day_group), "event",
				  G_CALLBACK (calendar_month_selection_changed),
				  cm);
	}
}

/*
 * Public Methods
 */

GtkWidget *
calendar_month_new (void)
{
	CalendarMonth *cm;

	cm = g_object_new (CALENDAR_TYPE_MONTH, NULL);

	return GTK_WIDGET (cm);
}

void
calendar_month_set_date (CalendarMonth *cm, const struct tm *tm)
{
	CalendarMonthPriv *priv;

	gboolean changed;

	g_return_if_fail (cm != NULL);
	g_return_if_fail (CALENDAR_IS_MONTH (cm));
	g_return_if_fail (tm != NULL);

	priv = cm->priv;

	changed =
		priv->year  != tm->tm_year + 1900 ||
		priv->month != tm->tm_mon  + 1    ||
		priv->day   != tm->tm_mday;

	priv->year  = tm->tm_year + 1900;
	priv->month = tm->tm_mon  + 1;
	priv->day   = tm->tm_mday;

	if (changed) {
		calendar_month_update (cm);

		g_signal_emit (G_OBJECT (cm), signals[SIGNAL_SELECTION_CHANGED], 0);
	}
}

void
calendar_month_get_date (CalendarMonth *cm, struct tm *tm)
{
	CalendarMonthPriv *priv;

	g_return_if_fail (cm != NULL);
	g_return_if_fail (CALENDAR_IS_MONTH (cm));
	g_return_if_fail (tm != NULL);

	priv = cm->priv;

	memset (tm, 0, sizeof (struct tm));

	tm->tm_year = priv->year - 1900;
	tm->tm_mon  = priv->month - 1;
	tm->tm_mday = priv->day;
}
