/*
 * EventEditor widget
 * Copyright (C) 1998 the Free Software Foundation
 *
 * Authors: Miguel de Icaza (miguel@kernel.org)
 *          Federico Mena (quartic@gimp.org)
 */

#include <config.h>

#include <string.h>

#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <gnome.h>

#include "calendar.h"
#include "eventedit.h"
#include "gnomecal-config.h"
#include "timeutil.h"
#include "utils-ui.h"


/* FIXME: ical->new must be set to FALSE in any case */
/* FIXME: instant apply */

struct _EventEditorPriv {
	/* The associated iCal object */

	iCalObject *ical;

	/* General widgets */

	GtkWidget *start_time;
	GtkWidget *end_time;

	/* Reccurence widgets */

	GtkWidget *recur_end_on;

	GtkWidget *recur_ex_date;

	/* Cached from gconf */

	gboolean week_starts_sunday;
	gboolean am_pm_flag;
	gint day_begin, day_end;

	/* Signal IDs */

	guint notify1, notify2, notify3, notify4;
};


static void event_editor_class_init	(EventEditorClass	*klass);
static void event_editor_init		(EventEditor		*ee);
static void event_editor_dispose	(GObject		*object);
static void event_editor_finalize	(GObject		*object);

/* FIXME: defined in main.c */
void save_default_calendar (GnomeCalendar *gcal);

GtkWidget* make_spin_button (int val, int low, int high);
void ee_create_ae (GtkTable *table, char *str, CalendarAlarm *alarm, enum AlarmType type, 
		   int y, gboolean control_sens, GtkSignalFunc dirty_func);
void ee_store_alarm (CalendarAlarm *alarm, enum AlarmType type);

/* Note: do not i18n these strings, they are part of the vCalendar protocol */
static char *class_names [] = { "PUBLIC", "PRIVATE", "CONFIDENTIAL" };

static GtkDialogClass *parent_class;

struct numbered_item {
	char *text;
	int num;
};

enum {
	EVEDIT_COLUMN_STRING,
	EVEDIT_COLUMN_TIME,
	EVEDIT_COLUMNS
};


GType
event_editor_get_type (void)
{
	static GType event_editor_type = 0;

	if (!event_editor_type) {
		static const GTypeInfo event_editor_info = {
			sizeof (EventEditorClass),
			NULL, /* base_init */
			NULL, /* base_finalize */
			(GClassInitFunc) event_editor_class_init,
			NULL, /* class_finalize */
			NULL, /* class_data */
			sizeof (EventEditor),
			3,    /* n_preallocs */
			(GInstanceInitFunc) event_editor_init,
		};

		event_editor_type = g_type_register_static (GTK_TYPE_DIALOG,
							    "EventEditor",
							    &event_editor_info,
							    0);
	}

	return event_editor_type;
}

static void
event_editor_class_init (EventEditorClass *klass)
{
	GObjectClass *gobject_class;

	parent_class = g_type_class_peek_parent (klass);

	gobject_class = G_OBJECT_CLASS (klass);

	gobject_class->dispose  = event_editor_dispose;
	gobject_class->finalize = event_editor_finalize;
}

GtkWidget *
make_spin_button (int val, int low, int high)
{
	GtkAdjustment *adj;
	GtkWidget *spin;

	adj = GTK_ADJUSTMENT (gtk_adjustment_new (val, low, high, 1, 10, 10));
	spin = gtk_spin_button_new (adj, 0.5, 0);
	gtk_widget_set_size_request (spin, 60, -1);

	return spin;
}

/*
 * Checks if the day range occupies all the day, and if so, check the
 * box accordingly
 */
static void
ee_check_all_day (EventEditor *ee)
{
	if (ee->priv->ical->allday) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ee->general_allday), 1);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ee->general_allday), 0);
	}
}

/*
 * Callback: checks that the dates are start < end
 */
static void
check_dates (GnomeDateEdit *gde, EventEditor *ee)
{
	time_t start, end;
	struct tm tm_start, tm_end;

	start = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->start_time));
	end   = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->end_time));

	if (start > end) {
		tm_start = *localtime (&start);
		tm_end = *localtime (&end);

		if (GTK_WIDGET (gde) == ee->priv->start_time) {
			tm_end.tm_year = tm_start.tm_year;
			tm_end.tm_mon  = tm_start.tm_mon;
			tm_end.tm_mday = tm_start.tm_mday;

			gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->end_time),
						  mktime (&tm_end));
		} else if (GTK_WIDGET (gde) == ee->priv->end_time) {
			tm_start.tm_year = tm_end.tm_year;
			tm_start.tm_mon  = tm_end.tm_mon;
			tm_start.tm_mday = tm_end.tm_mday;

			gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->start_time),
						  mktime (&tm_start));
		}
	}
}

/*
 * Callback: checks that start_time < end_time and whether the
 * selected hour range spans all of the day
 */
static void
check_times (GnomeDateEdit *gde, EventEditor *ee)
{
	time_t start, end;
	struct tm tm_start, tm_end;

	gdk_pointer_ungrab (GDK_CURRENT_TIME);
	gdk_flush ();

	start = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->start_time));
	end   = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->end_time));

	if (start >= end) {
		tm_start = *localtime (&start);
		tm_end = *localtime (&end);

		if (GTK_WIDGET (gde) == ee->priv->start_time) {
			tm_end.tm_min  = tm_start.tm_min;
			tm_end.tm_sec  = tm_start.tm_sec;

			tm_end.tm_hour = tm_start.tm_hour + 1;

			if (tm_end.tm_hour >= 24) {
				tm_end.tm_hour = 24; /* mktime() will bump the day */
				tm_end.tm_min = 0;
				tm_end.tm_sec = 0;
			}

			gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->end_time),
						  mktime (&tm_end));
		} else if (GTK_WIDGET (gde) == ee->priv->end_time) {
			tm_start.tm_min  = tm_end.tm_min;
			tm_start.tm_sec  = tm_end.tm_sec;

			tm_start.tm_hour = tm_end.tm_hour - 1;

			if (tm_start.tm_hour < 0) {
				tm_start.tm_hour = 0;
				tm_start.tm_min = 0;
				tm_start.tm_min = 0;
			}

			gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->start_time),
						  mktime (&tm_start));
		}
	}

	/* Check whether the event spans the whole day */
	ee_check_all_day (ee);
}

/*
 * Callbacks: called whenever GConf signals a configuration change
 */
static void
event_edit_general_config_changed (GConfClient	*client,
				   guint	 cnxn_id,
				   GConfEntry	*entry,
				   gpointer	 data)
{
	EventEditor *ee;
	GnomeDateEditFlags flag = 0;
	GnomeDateEditFlags flags;
	const gchar *key;
	gboolean v;

	g_return_if_fail (entry != NULL);
	g_return_if_fail (data != NULL);
	g_return_if_fail (IS_EVENT_EDITOR (data));

	ee = EVENT_EDITOR (data);

	v = gconf_value_get_bool (gconf_entry_get_value (entry));

	key = gconf_entry_get_key (entry);

	if (strcmp (key, CONFIG_GENERAL_AM_PM) == 0)
		flag |= GNOME_DATE_EDIT_24_HR;
	else if (strcmp (key, CONFIG_GENERAL_WEEK_START) == 0)
		flag |= GNOME_DATE_EDIT_WEEK_STARTS_ON_MONDAY;

	/* Get the flags from one of the date edits. The flags of all
	 * date edits should be equal anyway.
	 */
	flags = gnome_date_edit_get_flags (GNOME_DATE_EDIT (ee->priv->start_time));

	if (v)
		flags &= ~flag;
	else
		flags |= flag;

	gnome_date_edit_set_flags (GNOME_DATE_EDIT (ee->priv->start_time),    flags);
	gnome_date_edit_set_flags (GNOME_DATE_EDIT (ee->priv->end_time),      flags);
	gnome_date_edit_set_flags (GNOME_DATE_EDIT (ee->priv->recur_end_on),  flags);
	gnome_date_edit_set_flags (GNOME_DATE_EDIT (ee->priv->recur_ex_date), flags);
}

/*
 * Callbacks: called whenever GConf signals a configuration change
 */
static void
event_edit_calendar_config_changed (GConfClient	*client,
				    guint	 cnxn_id,
				    GConfEntry	*entry,
				    gpointer	 data)
{
	EventEditor *ee;
	const gchar *key;
	gint v;

	g_return_if_fail (entry != NULL);
	g_return_if_fail (data != NULL);
	g_return_if_fail (IS_EVENT_EDITOR (data));

	ee = EVENT_EDITOR (data);

	v = gconf_value_get_int (gconf_entry_get_value (entry));

	key = gconf_entry_get_key (entry);

	if (strcmp (key, CONFIG_CAL_DAY_START) == 0)
		ee->priv->day_begin = v;
	else if (strcmp (key, CONFIG_GENERAL_WEEK_START) == 0)
		ee->priv->day_end = v;

	/* Update the hour choosers */

	gnome_date_edit_set_popup_range (GNOME_DATE_EDIT (ee->priv->start_time),
					 ee->priv->day_begin, ee->priv->day_end);
	gnome_date_edit_set_popup_range (GNOME_DATE_EDIT (ee->priv->end_time),
					 ee->priv->day_begin, ee->priv->day_end);
}

/*
 * Callback: all day event box clicked
 */
static void
set_all_day (GtkToggleButton *toggle, EventEditor *ee)
{
	struct tm tm;
	time_t start_t;

	ee->priv->ical->allday = FALSE;
	
	if (toggle->active) {
		start_t = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->start_time));
		tm = *localtime (&start_t);
		tm.tm_hour = ee->priv->day_begin;
		tm.tm_min  = 0;
		tm.tm_sec  = 0;
		gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->start_time),
					  mktime (&tm));
		tm.tm_hour = ee->priv->day_end;
		gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->end_time),
					  mktime (&tm));
		ee->priv->ical->allday = TRUE;
	}
	ee_check_all_day (ee);
}

static GtkWidget *
event_editor_setup_time_frame (EventEditor *ee)
{
	GtkWidget *frame;
	GtkTable  *t;
	GnomeDateEditFlags date_flags;

	date_flags = GNOME_DATE_EDIT_SHOW_TIME;
	if (!ee->priv->week_starts_sunday)
		date_flags |= GNOME_DATE_EDIT_WEEK_STARTS_ON_MONDAY;
	if (!ee->priv->am_pm_flag)
		date_flags |= GNOME_DATE_EDIT_24_HR;

	frame = gtk_frame_new (_("Time"));
	t = GTK_TABLE (ee->general_time_table = gtk_table_new (1, 1, 0));
	gtk_container_set_border_width (GTK_CONTAINER (t), 4);
	gtk_table_set_row_spacings (t, 4);
	gtk_table_set_col_spacings (t, 4);
	gtk_container_add (GTK_CONTAINER (frame), ee->general_time_table);

	/* 1. Start time */
	if (ee->priv->ical->dtstart == 0){
		ee->priv->ical->dtstart = time (NULL);
		ee->priv->ical->dtend   = time_add_minutes (ee->priv->ical->dtstart, 30);
	}
	ee->priv->start_time = gnome_date_edit_new_flags (ee->priv->ical->dtstart,
							  date_flags);
	gnome_date_edit_set_popup_range (GNOME_DATE_EDIT (ee->priv->start_time),
					 ee->priv->day_begin, ee->priv->day_end);
	g_signal_connect (G_OBJECT (ee->priv->start_time), "date_changed",
			  G_CALLBACK (check_dates), ee);
	g_signal_connect (G_OBJECT (ee->priv->start_time), "time_changed",
			  G_CALLBACK (check_times), ee);
	gtk_table_attach (t, gtk_label_new (_("Start time:")), 1, 2, 1, 2,
			  GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);
	gtk_table_attach (t, ee->priv->start_time, 2, 3, 1, 2,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	/* 2. End time */
	ee->priv->end_time = gnome_date_edit_new_flags (ee->priv->ical->dtend,
							date_flags);
	gnome_date_edit_set_popup_range (GNOME_DATE_EDIT (ee->priv->end_time),
					 ee->priv->day_begin, ee->priv->day_end);
	g_signal_connect (G_OBJECT (ee->priv->end_time), "date_changed",
			  G_CALLBACK (check_dates), ee);
	g_signal_connect (G_OBJECT (ee->priv->end_time), "time_changed",
			  G_CALLBACK (check_times), ee);
	gtk_table_attach (t, gtk_label_new (_("End time:")), 1, 2, 2, 3,
			  GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);
	gtk_table_attach (t, ee->priv->end_time, 2, 3, 2, 3,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	/* 3. All day checkbox */
	ee->general_allday = gtk_check_button_new_with_label (_("All day event"));
	g_signal_connect (G_OBJECT (ee->general_allday), "toggled",
			  G_CALLBACK (set_all_day), ee);
	gtk_table_attach (t, ee->general_allday, 3, 4, 1, 2,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  4, 0);
	ee_check_all_day (ee);

	return frame;
}

static GtkWidget *
timesel_new (void)
{
	GtkWidget *menu, *option_menu;
	char *items [] = { N_("Minutes"), N_("Hours"), N_("Days") };
	int i;
	
	option_menu = gtk_option_menu_new ();
	menu = gtk_menu_new ();
	for (i = 0; i < 3; i++){
		GtkWidget *item;

		item = gtk_menu_item_new_with_label (_(items [i]));
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		gtk_widget_show (item);
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
	return option_menu;
}

/*
 * Set the sensitive state depending on whether the alarm enabled flag.
 */
static void
ee_alarm_setting (CalendarAlarm *alarm, int sensitive)
{
	gtk_widget_set_sensitive (GTK_WIDGET (alarm->w_count), sensitive);
	gtk_widget_set_sensitive (GTK_WIDGET (alarm->w_timesel), sensitive);

	if (alarm->type == ALARM_PROGRAM || alarm->type == ALARM_MAIL){
		gtk_widget_set_sensitive (GTK_WIDGET (alarm->w_entry), sensitive);
		gtk_widget_set_sensitive (GTK_WIDGET (alarm->w_label), sensitive);
	}
}

static void
alarm_toggle (GtkToggleButton *toggle, CalendarAlarm *alarm)
{
	ee_alarm_setting (alarm, toggle->active);
}

#define FXS (GTK_FILL | GTK_EXPAND | GTK_SHRINK)
#define FS  (GTK_FILL | GTK_SHRINK)

void
ee_create_ae (GtkTable *table, char *str, CalendarAlarm *alarm, enum AlarmType type, int y, gboolean control_sens, GtkSignalFunc dirty_func)
{
	GtkWidget *entry;

	alarm->w_enabled = gtk_check_button_new_with_label (str);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (alarm->w_enabled), 
				      alarm->enabled);
	if (control_sens)
		g_signal_connect (G_OBJECT (alarm->w_enabled), "toggled",
				  G_CALLBACK (alarm_toggle), alarm);
	if (dirty_func)
		g_signal_connect (G_OBJECT (alarm->w_enabled), "toggled",
				  G_CALLBACK (dirty_func), NULL);
	gtk_table_attach (table, alarm->w_enabled, 0, 1, y, y+1, FS, FS, 0, 0);

	alarm->w_count = make_spin_button (alarm->count, 0, 10000);
	if (dirty_func)
		g_signal_connect (G_OBJECT (alarm->w_count), "changed",
				  G_CALLBACK (dirty_func), NULL);
	gtk_table_attach (table, alarm->w_count, 1, 2, y, y+1, FS, FS, 0, 0);
	
	alarm->w_timesel = timesel_new ();
	/* is there a "changed" signal which we can connect to? */
	gtk_option_menu_set_history (GTK_OPTION_MENU (alarm->w_timesel), alarm->units);
	gtk_table_attach (table, alarm->w_timesel, 2, 3, y, y+1, FS, FS, 0, 0);

	if (dirty_func) {
		g_signal_connect (G_OBJECT (gtk_option_menu_get_menu(alarm->w_timesel)), 
				  "selection-done", 
				  G_CALLBACK (dirty_func), NULL);
	}

	switch (type){
	case ALARM_MAIL:
		alarm->w_label = gtk_label_new (_("Mail to:"));
		gtk_misc_set_alignment (GTK_MISC (alarm->w_label), 1.0, 0.5);	
		gtk_table_attach (table, alarm->w_label, 3, 4, y, y+1, FS, FS, 0, 0);
		alarm->w_entry = gtk_entry_new ();
		gtk_table_attach (table, alarm->w_entry, 4, 5, y, y+1, FXS, FS, 0, 0);
		gtk_entry_set_text (GTK_ENTRY (alarm->w_entry), alarm->data ? alarm->data : "");
		if (dirty_func)
			g_signal_connect (G_OBJECT (alarm->w_entry), "changed",
					  G_CALLBACK (dirty_func), NULL);
		break;

	case ALARM_PROGRAM:
		alarm->w_label = gtk_label_new (_("Run program:"));
		gtk_misc_set_alignment (GTK_MISC (alarm->w_label), 1.0, 0.5);	
		gtk_table_attach (table, alarm->w_label, 3, 4, y, y+1, FS, FS, 0, 0);
		alarm->w_entry = gnome_file_entry_new ("alarm-program", _("Select program to run at alarm time"));
		entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (alarm->w_entry));
		gtk_entry_set_text (GTK_ENTRY (entry), alarm->data ? alarm->data : "");
		gtk_table_attach (table, alarm->w_entry, 4, 5, y, y+1, FXS, FS, 0, 0);
		if (dirty_func)
			g_signal_connect (G_OBJECT (entry), "changed",
					  G_CALLBACK (dirty_func), NULL);
		break;

	default:
		break;
	}

	if (control_sens)
		ee_alarm_setting (alarm, alarm->enabled);
	else
		ee_alarm_setting (alarm, TRUE);
}

static GtkWidget *
ee_alarm_widgets (EventEditor *ee)
{
	GtkWidget *table, *mailto, *mailte, *l;
	
	l = gtk_frame_new (_("Alarms"));
	
	table = gtk_table_new (1, 1, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table), 4);
	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
	gtk_container_add (GTK_CONTAINER (l), table);
	
	mailto  = gtk_label_new (_("Mail to:"));
	mailte  = gtk_entry_new ();

	ee_create_ae (GTK_TABLE (table), _("Display"), &ee->priv->ical->dalarm, ALARM_DISPLAY, 1, TRUE, NULL);
	ee_create_ae (GTK_TABLE (table), _("Audio"),   &ee->priv->ical->aalarm, ALARM_AUDIO, 2, TRUE, NULL);
	ee_create_ae (GTK_TABLE (table), _("Program"), &ee->priv->ical->palarm, ALARM_PROGRAM, 3, TRUE, NULL);
	ee_create_ae (GTK_TABLE (table), _("Mail"),    &ee->priv->ical->malarm, ALARM_MAIL, 4, TRUE, NULL);
	
	return l;
}

static GtkWidget *
ee_classification_widgets (EventEditor *ee)
{
	GtkWidget *rpub, *rpriv, *rconf;
	GtkWidget *frame, *hbox;

	frame = gtk_frame_new (_("Classification"));

	hbox  = gtk_hbox_new (TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
	gtk_container_add (GTK_CONTAINER (frame), hbox);
	
	rpub  = gtk_radio_button_new_with_label (NULL, _("Public"));
	rpriv = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rpub), _("Private"));
	rconf  = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (rpub), _("Confidential"));

	gtk_box_pack_start (GTK_BOX (hbox), rpub, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), rpriv, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), rconf, FALSE, FALSE, 0);

	if (strcmp (ee->priv->ical->class, class_names[0]) == 0)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rpub), TRUE);
	else if (strcmp (ee->priv->ical->class, class_names[1]) == 0)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rpriv), TRUE);
	else if (strcmp (ee->priv->ical->class, class_names[2]) == 0)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rconf), TRUE);

	ee->general_radios = rpub;

	return frame;
}

/*
 * Retrieves the information from the CalendarAlarm widgets and stores them
 * on the CalendarAlarm generic values
 */
void
ee_store_alarm (CalendarAlarm *alarm, enum AlarmType type)
{
	GtkWidget *item;
	GtkMenu   *menu;
	GList     *child;
	int idx;
	
	if (alarm->data){
		g_free (alarm->data);
		alarm->data = 0;
	}
	
	alarm->enabled = GTK_TOGGLE_BUTTON (alarm->w_enabled)->active;

	if (!alarm->enabled)
		return;
	
	if (type == ALARM_PROGRAM)
		alarm->data = g_strdup (gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (alarm->w_entry))));
	if (type == ALARM_MAIL)
		alarm->data = g_strdup (gtk_entry_get_text (GTK_ENTRY (alarm->w_entry)));

	/* Find out the index */
	menu = GTK_MENU (GTK_OPTION_MENU (alarm->w_timesel)->menu);
	
	item = gtk_menu_get_active (menu);
	
	for (idx = 0, child = GTK_MENU_SHELL (menu)->children; child->data != item; child = child->next)
		idx++;
	
	alarm->units = idx;
	alarm->count = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (alarm->w_count));
}

static void
ee_store_general_values_to_ical (EventEditor *ee)
{
	GtkRadioButton *radio = GTK_RADIO_BUTTON (ee->general_radios);
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	iCalObject *ical = ee->priv->ical;
	GSList *list = radio->group;
	int idx;
	time_t now;

	now = time (NULL);
	ical->dtstart = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->start_time));
	ical->dtend   = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->end_time));

	if (ical->summary)
		g_free (ical->summary);

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (ee->general_summary));

	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_get_end_iter (buffer, &end);
	ical->summary = gtk_text_buffer_get_text (buffer,
						  &start, &end,
						  FALSE);

	ee_store_alarm (&ical->dalarm, ALARM_DISPLAY);
	ee_store_alarm (&ical->aalarm, ALARM_AUDIO);
	ee_store_alarm (&ical->palarm, ALARM_PROGRAM);
	ee_store_alarm (&ical->malarm, ALARM_MAIL);

	for (idx = 2; list; list = list->next) {
		if (GTK_TOGGLE_BUTTON (list->data)->active)
			break;
		idx--; /* The group is stored in reverse order of insertion */
	}

	g_free (ical->class);
	ical->class = g_strdup (class_names [idx]);
}

static int
option_menu_active_number (GtkWidget *omenu)
{
	GtkWidget *menu;
	GtkWidget *item;
	struct numbered_item *ni;

	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (omenu));
	item = gtk_menu_get_active (GTK_MENU (menu));

	ni = g_object_get_data (G_OBJECT (item), "numbered_item");

	return ni->num;
}

static int
ee_store_recur_rule_to_ical (EventEditor *ee)
{
	iCalObject *ical;
	int i, j;
	GSList *list;

	ical = ee->priv->ical;

	for (i = 0, list = ee->recur_rr_group; list; i++, list = list->next)
		if (GTK_TOGGLE_BUTTON (list->data)->active)
			break;

	i = g_slist_length (ee->recur_rr_group) - i - 1; /* buttons are stored in reverse order of insertion */

	/* None selected, no rule to be stored */
	if (i == 0)
		return 0;

	if (!ical->recur)
		ical->recur = g_new0 (Recurrence, 1);
	
	switch (i) {
	case 1:
		/* Daily */
		ical->recur->type = RECUR_DAILY;
		ical->recur->interval = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ee->recur_rr_day_period));
		break;

	case 2:
		/* Weekly */
		ical->recur->type = RECUR_WEEKLY;
		ical->recur->interval = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ee->recur_rr_week_period));
		ical->recur->weekday = 0;

		for (j = 0; j < 7; j++)
			if (GTK_TOGGLE_BUTTON (ee->recur_rr_week_days[j])->active) {
				if (j == 6)
					ical->recur->weekday |= 1 << 0; /* sunday is at bit 0 */
				else
					ical->recur->weekday |= 1 << (j + 1);
			}

		break;

	case 3:
		/* Monthly */

		if (GTK_WIDGET_SENSITIVE (ee->recur_rr_month_date)) {
			/* by day */

			ical->recur->type = RECUR_MONTHLY_BY_DAY;
			ical->recur->u.month_day =
				gtk_spin_button_get_value_as_int (
					GTK_SPIN_BUTTON (ee->recur_rr_month_date));
			ical->recur->interval =
				gtk_spin_button_get_value_as_int (
					GTK_SPIN_BUTTON (ee->recur_rr_month_period));
		} else {
			/* by position */

			ical->recur->type = RECUR_MONTHLY_BY_POS;

			ical->recur->u.month_pos =
				option_menu_active_number (ee->recur_rr_month_day);
			ical->recur->weekday =
				option_menu_active_number (ee->recur_rr_month_weekday);
			ical->recur->interval =
				gtk_spin_button_get_value_as_int (
					GTK_SPIN_BUTTON (ee->recur_rr_month_period));
		}

		break;

	case 4:
		/* Yearly */
		ical->recur->type = RECUR_YEARLY_BY_DAY;
		ical->recur->interval = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ee->recur_rr_year_period));
		/* FIXME: need to specify anything else?  I am assuming the code will look at the dtstart
		 * to figure out when to recur. - Federico
		 */
		break;

	default:
		g_assert_not_reached ();
	}
	return 1;
}

static void
ee_store_recur_end_to_ical (EventEditor *ee)
{
	iCalObject *ical;
	GSList *list;
	int i;

	/* Ending date of recurrence */

	ical = ee->priv->ical;

	for (i = 0, list = ee->recur_ed_group; list; i++, list = list->next)
		if (GTK_TOGGLE_BUTTON (list->data)->active)
			break;

	i = g_slist_length (ee->recur_ed_group) - i - 1; /* the list is stored in reverse order of insertion */

	switch (i) {
	case 0:
		/* repeat forever */
		ical->recur->_enddate = 0;
		ical->recur->enddate = 0;
		ical->recur->duration = 0;
		break;

	case 1:
		/* end date */
		/* Also here, to ensure that the event is used, we add 86400 secs to get 
		   get next day, in accordance to the RFC */
		ical->recur->_enddate = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->recur_end_on)) + 86400;
		ical->recur->enddate = ical->recur->_enddate;
		ical->recur->duration = 0;
		break;

	case 2:
		/* end after n occurrences */
		ical->recur->duration = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (ee->recur_ed_end_after));
		ical_object_compute_end (ical);
		break;

	default:
		g_assert_not_reached ();
		break;
	}
}

static void
free_exdate (iCalObject *ical)
{
	GList *list;

	if (!ical->exdate)
		return;

	for (list = ical->exdate; list; list = list->next)
		g_free (list->data);

	g_list_free (ical->exdate);
	ical->exdate = NULL;
}

static void
ee_store_recur_exceptions_to_ical (EventEditor *ee)
{
	GtkTreeModel *model;
	GtkTreeIter   iter;
	iCalObject   *ical;

	ical = ee->priv->ical;
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (ee->recur_ex_list));

	free_exdate (ical);

	gtk_tree_model_get_iter_root (model, &iter);
	if (gtk_tree_model_iter_children (model, &iter, &iter)) {
		time_t *t;

		gtk_tree_model_get (model, &iter,
				    1, &t, -1);
		ical->exdate = g_list_prepend (ical->exdate, t);

		gtk_tree_model_iter_next (model, &iter);
	}
}

static void
ee_store_recur_values_to_ical (EventEditor *ee)
{
	if (ee_store_recur_rule_to_ical (ee)){
		ee_store_recur_exceptions_to_ical (ee);
		ee_store_recur_end_to_ical (ee);
	} else if (ee->priv->ical->recur) {
		g_free (ee->priv->ical->recur);
		ee->priv->ical->recur = NULL;
		if (ee->priv->ical->exdate){
			GList *l = ee->priv->ical->exdate;

			for (; l; l = l->next)
				g_free (l->data);
			g_list_free (l);
		}
	}
}

/*
 * Retrieves all of the information from the different widgets and updates
 * the iCalObject accordingly.
 */
static void
ee_store_dlg_values_to_ical (EventEditor *ee)
{
	time_t now;

	ee_store_general_values_to_ical (ee);
	ee_store_recur_values_to_ical (ee);

	now = time (NULL);

	/* FIXME: This is not entirely correct; we should check if the values actually changed */
	ee->priv->ical->last_mod = now;

	if (ee->priv->ical->new)
		ee->priv->ical->created = now;
}

/* FIXME: will get obsolete with instant apply */
static gboolean
event_editor_save_object (EventEditor *ee)
{
	time_t start, end;

	start = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->start_time));
	end   = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->end_time));

	if (start >= end) {
		GtkWidget *mbox;
		mbox = gtk_message_dialog_new (GTK_WINDOW (ee),
					       GTK_DIALOG_DESTROY_WITH_PARENT,
					       GTK_MESSAGE_ERROR,
					       GTK_BUTTONS_OK,
					       _("An event's end time must be later than its start time."));
		g_signal_connect (G_OBJECT (mbox), "response",
				  G_CALLBACK (gtk_widget_destroy), NULL);
		gtk_widget_show (mbox);
		return FALSE;
	}

	ee_store_dlg_values_to_ical (ee);

	if (ee->priv->ical->new)
		gnome_calendar_add_object (ee->gnome_cal, ee->priv->ical);
	else
		gnome_calendar_object_changed (ee->gnome_cal, ee->priv->ical, CHANGE_ALL);

	save_default_calendar (ee->gnome_cal);
	ee->priv->ical->new = 0;

	return TRUE;
}

static void
event_editor_response (GtkDialog *dialog, gint response_id, gpointer data)
{
	g_return_if_fail (dialog != NULL);
	g_return_if_fail (IS_EVENT_EDITOR (dialog));

	switch (response_id) {
	/* FIXME: the next two statements will be removed, once instant apply is in effect */
	case GTK_RESPONSE_OK:
		if (!event_editor_save_object (EVENT_EDITOR (dialog)))
			break;
		/* fall through */
	case GTK_RESPONSE_CANCEL:
		/* fall through */

	case GTK_RESPONSE_CLOSE:
		gtk_widget_destroy (GTK_WIDGET (dialog));
		break;
	case GTK_RESPONSE_DELETE_EVENT:
		break;
	case GTK_RESPONSE_HELP:
		ui_show_help (GTK_WINDOW (dialog), "event-editor");
		break;
	default:
		g_return_if_reached ();
	}
}

/*
 * Load the contents in a delayed fashion, as the GtkText widget needs it
 */
static void
ee_fill_summary (GtkWidget *widget, EventEditor *ee)
{
	GtkTextBuffer *buffer;

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (ee->general_summary));
	gtk_text_buffer_set_text (buffer, ee->priv->ical->summary, -1);
}

enum {
	OWNER_LINE,
	DESC_LINE,
	SUMMARY_LINE,
	TIME_LINE,
	ALARM_LINE,
	CLASS_LINE
};

/* Create/setup the general page */
static GtkWidget *
event_editor_create_general_page (EventEditor *ee)
{
	GtkWidget *page;
	GtkWidget *l;
	GtkWidget *hbox;
	GtkWidget *frame;

	page = gtk_table_new (1, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (page), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (page), GNOME_PAD_SMALL);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_table_attach (GTK_TABLE (page), hbox,
			  0, 1, OWNER_LINE, OWNER_LINE + 1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 4);
	
	l = gtk_label_new (_("Owner:"));
	gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);

	ee->general_owner = gtk_label_new (ee->priv->ical->organizer ? ee->priv->ical->organizer : _("?"));
	gtk_misc_set_alignment (GTK_MISC (ee->general_owner), 0.0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), ee->general_owner, TRUE, TRUE, 4);

	l = gtk_label_new (_("Summary:"));
	gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE (page), l,
			  0, 1, DESC_LINE, DESC_LINE + 1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
	ee->general_summary = gtk_text_view_new ();
	g_signal_connect (G_OBJECT (ee->general_summary), "realize",
			  G_CALLBACK (ee_fill_summary), ee);
	gtk_widget_set_size_request (ee->general_summary, 0, 60);
	gtk_text_view_set_editable  (GTK_TEXT_VIEW (ee->general_summary), TRUE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (ee->general_summary), GTK_WRAP_WORD);
	gtk_container_add (GTK_CONTAINER (frame), ee->general_summary);
	gtk_table_attach (GTK_TABLE (page), frame,
			  0, 1, SUMMARY_LINE, SUMMARY_LINE+1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	l = ee_alarm_widgets (ee);
	gtk_table_attach (GTK_TABLE (page), l,
			  0, 1, ALARM_LINE, ALARM_LINE + 1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	l = event_editor_setup_time_frame (ee);
	gtk_table_attach (GTK_TABLE (page), l,
			  0, 1, TIME_LINE, TIME_LINE + 1,
			  GTK_EXPAND | GTK_FILL | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	l = ee_classification_widgets (ee);
	gtk_table_attach (GTK_TABLE (page), l,
			  0, 1, CLASS_LINE, CLASS_LINE + 1,
			  GTK_EXPAND | GTK_SHRINK,
			  GTK_FILL | GTK_SHRINK,
			  0, 0);

	return page;
}

struct {
	char *name;
} recurrence_types [] = {
	{ N_("None") },
	{ N_("Daily") },
	{ N_("Weekly") },
	{ N_("Monthly") },
	{ N_("Yearly") },
	{ 0 }
};

static void
recurrence_toggled (GtkRadioButton *radio, EventEditor *ee)
{
	GSList *list = ee->recur_rr_group;
	int which;

	if (!GTK_TOGGLE_BUTTON (radio)->active)
		return;

	for (which = 0; list; list = list->next, which++) {
		if (list->data == radio) {
			gtk_notebook_set_current_page (GTK_NOTEBOOK (ee->recur_rr_notebook),
						       4 - which);
			return;
		}
	}
}

static struct numbered_item weekday_positions[] = {
	{ N_("1st"), 1 },
	{ N_("2nd"), 2 },
	{ N_("3rd"), 3 },
	{ N_("4th"), 4 },
	{ N_("5th"), 5 },
	{ 0 }
};

static struct numbered_item weekday_names[] = {
	{ N_("Monday"),    1 },
	{ N_("Tuesday"),   2 },
	{ N_("Wednesday"), 3 },
	{ N_("Thursday"),  4 },
	{ N_("Friday"),    5 },
	{ N_("Saturday"),  6 },
	{ N_("Sunday"),    0 }, /* on the spec, Sunday is zero */
	{ 0 }
};

static GtkWidget *
make_numbered_menu (struct numbered_item *items, int sel)
{
	GtkWidget *option_menu, *menu;
	int i, history = 0;
        
	option_menu = gtk_option_menu_new ();
	menu = gtk_menu_new ();

	for (i = 0; items[i].text; i++) {
		GtkWidget *item;

		item = gtk_menu_item_new_with_label (_(items[i].text));
		g_object_set_data (G_OBJECT (item), "numbered_item", &items[i]);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		gtk_widget_show (item);

                if (items[i].num == sel)
                    history = i;
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), history);

	return option_menu;
}

static void
month_sensitize (EventEditor *ee, int state)
{
	gtk_widget_set_sensitive (ee->recur_rr_month_date, state);
	gtk_widget_set_sensitive (ee->recur_rr_month_date_label, state);
	
	gtk_widget_set_sensitive (ee->recur_rr_month_day,  !state);
	gtk_widget_set_sensitive (ee->recur_rr_month_weekday,  !state);
}

static void
recur_month_enable_date (GtkToggleButton *button, EventEditor *ee)
{
	month_sensitize (ee, button->active);
}

static void
desensitize_on_toggle (GtkToggleButton *toggle, gpointer data)
{
	gtk_widget_set_sensitive (GTK_WIDGET (data), !toggle->active);
}

static void
ee_rp_init_rule (EventEditor *ee)
{
	static char *day_names [] = { N_("Mon"), N_("Tue"), N_("Wed"), N_("Thu"), N_("Fri"), N_("Sat"), N_("Sun") };
	GtkWidget   *r, *re, *r1, *f, *vbox, *hbox, *b, *week_hbox, *week_day, *w;
	GtkWidget   *none, *daily, *weekly, *monthly, *yearly;
	GtkNotebook *notebook;
	GSList      *group;
	int          i, page, day_period, week_period, month_period, year_period;
	int          week_vector, default_day, def_pos, def_off;
	struct tm   tm;

	tm = *localtime (&ee->priv->ical->dtstart);
	
	f = gtk_frame_new (_("Recurrence Rule"));

	hbox = gtk_hbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
	gtk_container_add (GTK_CONTAINER (f), hbox);

	vbox = gtk_vbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (hbox), gtk_vseparator_new (), FALSE, FALSE, 0);

	ee->recur_rr_notebook = gtk_notebook_new ();
	notebook = GTK_NOTEBOOK (ee->recur_rr_notebook);
	gtk_container_set_border_width (GTK_CONTAINER (notebook),
					GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (hbox), ee->recur_rr_notebook, TRUE, TRUE, 0);

	day_period   = 1;
	week_period  = 1;
	month_period = 1;
	year_period  = 1;

	/* Default to today */

	week_vector = 1 << tm.tm_wday;
	default_day = tm.tm_mday;
	def_pos = 0;
	def_off = 0;

	/* Determine which should be the default selection */

	page = 0;
	if (ee->priv->ical->recur) {
		enum RecurType type = ee->priv->ical->recur->type;
		int interval = ee->priv->ical->recur->interval;

		switch (type) {
		case RECUR_DAILY:
			page = 1;
			day_period = interval;
			break;

		case RECUR_WEEKLY:
			page = 2;
			week_period = interval;
			week_vector = ee->priv->ical->recur->weekday;
			break;

		case RECUR_MONTHLY_BY_POS:
			page = 3;
			month_period = interval;
			def_pos = ee->priv->ical->recur->u.month_pos;
			default_day = ee->priv->ical->recur->weekday; /* you can't use u.month_pos and u.month_day-- it's a union... */
			break;

		case RECUR_MONTHLY_BY_DAY:
			page = 3;
			month_period = interval;
			default_day = ee->priv->ical->recur->u.month_day;
			break;

		case RECUR_YEARLY_BY_MONTH:
			page = 4;
			year_period  = interval;
			break;

		case RECUR_YEARLY_BY_DAY:
			page = 4;
			year_period  = interval;
			break;
		}
	} else
		page = 0;

	/* The recurrency selector */

	for (i = 0, group = NULL; recurrence_types [i].name; i++) {
		r = gtk_radio_button_new_with_label (group, _(recurrence_types [i].name));
		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (r));

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r), i == page);
		g_signal_connect (G_OBJECT (r), "toggled",
				  G_CALLBACK (recurrence_toggled), ee);
		gtk_box_pack_start (GTK_BOX (vbox), r, FALSE, FALSE, 0);

		if (i == 0)
			g_signal_connect (G_OBJECT (r), "toggled",
					  G_CALLBACK (desensitize_on_toggle),
					  ee->recur_hbox);
	}

	ee->recur_rr_group = group;

	/* 0. No recurrence */
	none = gtk_label_new ("");
	
	/* 1. The daily recurrence */

	daily = gtk_vbox_new (FALSE, 0);

	b = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (daily), b, FALSE, FALSE, 0);

	ee->recur_rr_day_period = make_spin_button (day_period, 1, 10000);
	/* In some languages "Every" can follow the gender of the word it
	   refers to (here "day(s)"). Leave the two leading letters "d_"
	   in the translation; they are there on purpose. */
	gtk_box_pack_start (GTK_BOX (b), gtk_label_new (_("d_Every") + 2), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (b), ee->recur_rr_day_period, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (b), gtk_label_new (_("day(s)")), FALSE, FALSE, 0);

	/* 2. The weekly recurrence */

	weekly = gtk_vbox_new (FALSE, 4);

	week_hbox = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (weekly), week_hbox, FALSE, FALSE, 0);

	/* 2.1 The week period selector */

	ee->recur_rr_week_period = make_spin_button (week_period, 1, 10000);
	/* In some languages "Every" can follow the gender of the word it
	   refers to (here "week(s)"). Leave the two leading letters "w_"
	   in the translation; they are there on purpose. */
	gtk_box_pack_start (GTK_BOX (week_hbox), gtk_label_new (_("w_Every") + 2), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (week_hbox), ee->recur_rr_week_period, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (week_hbox), gtk_label_new (_("week(s)")), FALSE, FALSE, 0);

	/* 2.2 The week day selector */

	week_day = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (weekly), week_day, FALSE, FALSE, 0);

	for (i = 0; i < 7; i++) {
		ee->recur_rr_week_days [i] = gtk_check_button_new_with_label (_(day_names [i]));
		gtk_box_pack_start (GTK_BOX (week_day), ee->recur_rr_week_days [i], FALSE, FALSE, 0);

		if (week_vector & (1 << ((i + 1) % 7))) /* on the spec, Sunday is 0 */
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ee->recur_rr_week_days [i]), TRUE);
	}

	/* 3. The monthly recurrence */

	monthly = gtk_table_new (0, 0, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (monthly), 4);
	gtk_table_set_col_spacings (GTK_TABLE (monthly), 4);

	re = gtk_radio_button_new_with_label (NULL, _("Recur on the"));
	ee->recur_rr_month_date = make_spin_button (default_day, 1, 31);
	ee->recur_rr_month_date_label = w = gtk_label_new (_("th day of the month"));
	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE (monthly), re,
			  0, 1, 0, 1, FS, FS, 0, 0);
	gtk_table_attach (GTK_TABLE (monthly), ee->recur_rr_month_date,
			  1, 2, 0, 1, FS, FS, 0, 0);
	gtk_table_attach (GTK_TABLE (monthly), w, 
			  2, 3, 0, 1, FS, FS, 0, 0);
	g_signal_connect (G_OBJECT (re), "toggled",
			  G_CALLBACK (recur_month_enable_date), ee);

	r1 = gtk_radio_button_new_with_label (gtk_radio_button_get_group (GTK_RADIO_BUTTON (re)),
					      _("Recur on the"));
	ee->recur_rr_month_day = make_numbered_menu (weekday_positions, def_pos);
	ee->recur_rr_month_weekday = make_numbered_menu (weekday_names, default_day);
	gtk_table_attach (GTK_TABLE (monthly), r1,
			  0, 1, 1, 2, FS, FS, 0, 0);
	gtk_table_attach (GTK_TABLE (monthly), ee->recur_rr_month_day,
			  1, 2, 1, 2, FS, FS, 0, 0);
	gtk_table_attach (GTK_TABLE (monthly), ee->recur_rr_month_weekday,
			  2, 3, 1, 2, FS, FS, 0, 0);

	/* in some languages "Every" can follow the gender of the word it
	   refers to (here "month(s)"). leave the two leadin letter "m_"
	   in the translation they are ther on purpose */
	gtk_table_attach (GTK_TABLE (monthly), gtk_label_new (_("m_Every") +2),
			  3, 4, 0, 2, FS, FS, 0, 0);
	ee->recur_rr_month_period = make_spin_button (month_period, 1, 10000);
	gtk_table_attach (GTK_TABLE (monthly), ee->recur_rr_month_period,
			  4, 5, 0, 2, FS, FS, 0, 0);
	gtk_table_attach (GTK_TABLE (monthly), gtk_label_new (_("month(s)")),
			  5, 6, 0, 2, FS, FS, 0, 0);

	if (ee->priv->ical->recur) {
		if (ee->priv->ical->recur->type == RECUR_MONTHLY_BY_POS)
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (r1), 1);
	} else
		recur_month_enable_date (GTK_TOGGLE_BUTTON (re), ee);

	/* 4. The yearly recurrence */

	yearly = gtk_vbox_new (FALSE, 0);

	b = gtk_hbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (yearly), b, FALSE, FALSE, 0);

	ee->recur_rr_year_period = make_spin_button (year_period, 1, 10000);
	/* in some languages "Every" can follow the gender of the word it
	   refers to (here "year(s)"). leave the two leadin letter "y_"
	   in the translation they are ther on purpose */
	gtk_box_pack_start (GTK_BOX (b), gtk_label_new (_("y_Every") +2), FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (b), ee->recur_rr_year_period, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (b), gtk_label_new (_("year(s)")), FALSE, FALSE, 0);

	/* Finish setting this up */

	gtk_notebook_append_page (notebook, none, gtk_label_new ("None"));
	gtk_notebook_append_page (notebook, daily, gtk_label_new ("Daily"));
	gtk_notebook_append_page (notebook, weekly, gtk_label_new ("Weekly"));
	gtk_notebook_append_page (notebook, monthly, gtk_label_new ("Monthly"));
	gtk_notebook_append_page (notebook, yearly, gtk_label_new ("Yearly"));
	gtk_notebook_set_show_tabs (notebook, FALSE);
	gtk_notebook_set_show_border (notebook, FALSE);

	gtk_widget_show_all (GTK_WIDGET (notebook));
	gtk_notebook_set_current_page (notebook, page);
	/* Attach to the main box */

	gtk_box_pack_start (GTK_BOX (ee->recur_vbox), f, FALSE, FALSE, 0);
}


static void
sensitize_on_toggle (GtkToggleButton *toggle, gpointer data)
{
	gtk_widget_set_sensitive (GTK_WIDGET (data), toggle->active);
}

static void
ee_rp_init_ending_date (EventEditor *ee)
{
	GtkWidget *frame;
	GtkWidget *vbox;
	GSList    *group;
	GtkWidget *radio0, *radio1, *radio2;
	GtkWidget *hbox;
	GtkWidget *ihbox;
	GtkWidget *widget;
	time_t     enddate;
	int        repeat;
	GnomeDateEditFlags date_flags;

	date_flags = 0;
	if (!ee->priv->week_starts_sunday)
		date_flags |= GNOME_DATE_EDIT_WEEK_STARTS_ON_MONDAY;
	if (!ee->priv->am_pm_flag)
		date_flags |= GNOME_DATE_EDIT_24_HR;

	frame = gtk_frame_new (_("Ending Date"));

	vbox = gtk_vbox_new (TRUE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	group = NULL;

	/* repeat forever */

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	radio0 = gtk_radio_button_new_with_label (group, _("Repeat forever"));
	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio0));
	gtk_box_pack_start (GTK_BOX (hbox), radio0, FALSE, FALSE, 0);

	/* end on date */

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	radio1 = gtk_radio_button_new_with_label (group, _("End on"));
	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio1));
	gtk_box_pack_start (GTK_BOX (hbox), radio1, FALSE, FALSE, 0);

	ihbox = gtk_hbox_new (FALSE, 4);
	gtk_widget_set_sensitive (ihbox, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), ihbox, FALSE, FALSE, 0);

	if (ee->priv->ical->recur) {
		/* Shorten by one day, as we store end-on date a day ahead */
		enddate = ee->priv->ical->recur->enddate - 86400;
	} else
		enddate = ee->priv->ical->dtend;

	ee->priv->recur_end_on = gnome_date_edit_new_flags (enddate,
							    date_flags);
	gtk_box_pack_start (GTK_BOX (ihbox),
			    ee->priv->recur_end_on, FALSE, FALSE, 0);

	g_signal_connect (G_OBJECT (radio1), "toggled",
			  G_CALLBACK (sensitize_on_toggle), ihbox);

	/* end after n occurrences */

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	radio2 = gtk_radio_button_new_with_label (group, _("End after"));
	group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio2));
	gtk_box_pack_start (GTK_BOX (hbox), radio2, FALSE, FALSE, 0);

	ihbox = gtk_hbox_new (FALSE, 4);
	gtk_widget_set_sensitive (ihbox, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), ihbox, FALSE, FALSE, 0);

	if (ee->priv->ical->recur && ee->priv->ical->recur->duration)
		repeat = ee->priv->ical->recur->duration;
	else
		repeat = 2;

	ee->recur_ed_end_after = widget = make_spin_button (repeat, 1, 10000);
	gtk_box_pack_start (GTK_BOX (ihbox), widget, FALSE, FALSE, 0);

	widget = gtk_label_new (_("occurrence(s)"));
	gtk_box_pack_start (GTK_BOX (ihbox), widget, FALSE, FALSE, 0);

	g_signal_connect (G_OBJECT (radio2), "toggled",
			  G_CALLBACK (sensitize_on_toggle), ihbox);

	/* Activate appropriate item */

	if (ee->priv->ical->recur) {
		if (ee->priv->ical->recur->_enddate == 0) {
			if (ee->priv->ical->recur->duration == 0)
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio0), TRUE);
			else {
				gtk_spin_button_set_value (GTK_SPIN_BUTTON (ee->recur_ed_end_after),
							   ee->priv->ical->recur->duration);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);
			}
		} else {
			gnome_date_edit_set_time (GNOME_DATE_EDIT (ee->priv->recur_end_on),
						  ee->priv->ical->recur->enddate-86400);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
		}
	}

	/* Done, add to main table */

	ee->recur_ed_group = group;

	gtk_box_pack_start (GTK_BOX (ee->recur_hbox), frame, FALSE, FALSE, 0);
}

static void
ee_append_exception (EventEditor *ee, time_t t)
{
	GtkTreeView  *tree;
	GtkTreeModel *model;
	GtkTreeIter   iter;
	time_t       *tt;
	gchar        *s;

	tree  = GTK_TREE_VIEW (ee->recur_ex_list);
	model = gtk_tree_view_get_model (tree);

	s  = time_get_date_string (t);
	tt = (time_t *) g_memdup (&t, sizeof (time_t));

	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
			    EVEDIT_COLUMN_STRING, s,
			    EVEDIT_COLUMN_TIME,   tt,
			    -1);
	gtk_tree_selection_select_iter (gtk_tree_view_get_selection (tree),
					&iter);

	g_free (s);

	gtk_widget_set_sensitive (ee->recur_ex_vbox, TRUE);
}

static void
ee_fill_exception_list (EventEditor *ee)
{
	GList  *list;

	for (list = ee->priv->ical->exdate; list; list = list->next)
		ee_append_exception (ee, *((time_t *) list->data));
}

static void
ee_add_exception (GtkWidget *widget, EventEditor *ee)
{
	time_t t;

	t = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->recur_ex_date));
	if (t >= 0)
		ee_append_exception (ee, t);
}

static void
ee_change_exception (GtkWidget *widget, EventEditor *ee)
{
	GtkTreeView  *tree;
	GtkTreeModel *model;
	GtkTreeIter   iter;
	time_t  t;
	time_t *tp;
	gchar  *s;

	tree = GTK_TREE_VIEW (ee->recur_ex_list);

	t = gnome_date_edit_get_time (GNOME_DATE_EDIT (ee->priv->recur_ex_date));
	if (t < 0)
		return;

	gtk_tree_selection_get_selected (gtk_tree_view_get_selection (tree),
					 &model, &iter);
	gtk_tree_model_get (model, &iter, 1, &tp, -1);
	*tp = t;

	s = time_get_date_string (t);
	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, s, -1);
	g_free (s);
}

static void
ee_delete_exception (GtkWidget *widget, EventEditor *ee)
{
	GtkTreeView      *tree;
	GtkTreeModel     *model;
	GtkTreeSelection *selection;
	GtkTreeIter       iter;
	time_t *t;

	tree = GTK_TREE_VIEW (ee->recur_ex_list);
	selection = gtk_tree_view_get_selection (tree);

	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
		return;

	gtk_tree_model_get (model, &iter, 1, &t, -1);
	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);

	g_free (t); /* free the time_t stored there */

	/* select next row (or last if this was the last) */
	if (iter.stamp == 0) {
		gint children;

		children = gtk_tree_model_iter_n_children (model, NULL);

		if (children == 0)
			gtk_widget_set_sensitive (ee->recur_ex_vbox, FALSE);
		else {
			gtk_tree_model_iter_nth_child   (model,
							 &iter, NULL,
							 children - 1);
			gtk_tree_selection_select_iter (selection, &iter);
		}

	} else
		gtk_tree_selection_select_iter (selection, &iter);
}

static void
ee_rp_init_exceptions (EventEditor *ee, GtkBox *box)
{
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *ivbox;
	GtkWidget *widget;
	GtkWidget *sw;
	GtkListStore    *store;
	GtkCellRenderer *renderer;
	GnomeDateEditFlags date_flags;

	date_flags = 0;
	if (!ee->priv->week_starts_sunday)
		date_flags |= GNOME_DATE_EDIT_WEEK_STARTS_ON_MONDAY;
	if (!ee->priv->am_pm_flag)
		date_flags |= GNOME_DATE_EDIT_24_HR;

	frame = gtk_frame_new (_("Exceptions"));
	gtk_widget_show (frame);
	gtk_box_pack_start (box, frame, TRUE, TRUE, 0);

	hbox = gtk_hbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
	gtk_container_add (GTK_CONTAINER (frame), hbox);

	vbox = gtk_vbox_new (FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

	ee->priv->recur_ex_date = gnome_date_edit_new_flags (0, date_flags);
	gtk_box_pack_start (GTK_BOX (vbox), ee->priv->recur_ex_date, FALSE, FALSE, 0);

	widget = gtk_button_new_with_label (_("Add exception"));
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (ee_add_exception), ee);
	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);

	ee->recur_ex_vbox = ivbox = gtk_vbox_new (FALSE, 4);
	gtk_widget_set_sensitive (ivbox, FALSE); /* at first there are no items to change or delete */
	gtk_box_pack_start (GTK_BOX (vbox), ivbox, FALSE, FALSE, 0);
	
	widget = gtk_button_new_with_label (_("Change selected"));
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (ee_change_exception), ee);
	gtk_box_pack_start (GTK_BOX (ivbox), widget, FALSE, FALSE, 0);

	widget = gtk_button_new_with_label (_("Delete selected"));
	g_signal_connect (G_OBJECT (widget), "clicked",
			  G_CALLBACK (ee_delete_exception), ee);
	gtk_box_pack_start (GTK_BOX (ivbox), widget, FALSE, FALSE, 0);

	/* TreeView */

	sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
					     GTK_SHADOW_IN);
	gtk_widget_show (sw);
	gtk_box_pack_start (GTK_BOX (hbox), sw, TRUE, TRUE, 0);

	store = gtk_list_store_new (EVEDIT_COLUMNS,
				    G_TYPE_STRING, G_TYPE_POINTER);
	ee_fill_exception_list (ee);

	ee->recur_ex_list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (ee->recur_ex_list),
					   FALSE);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (ee->recur_ex_list)),
				     GTK_SELECTION_BROWSE);
	g_object_unref (G_OBJECT (store));

	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (ee->recur_ex_list),
						     -1,
						     _("Exceptions"),
						     renderer,
						     "text", 0,
						     NULL);

	gtk_widget_show (ee->recur_ex_list);
	gtk_container_add (GTK_CONTAINER (sw), ee->recur_ex_list);
}

static GtkWidget *
event_editor_create_recurrence_page (EventEditor *ee)
{
	ee->recur_vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);

	ee->recur_hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_widget_set_sensitive (ee->recur_hbox, FALSE);

	ee_rp_init_rule (ee);

	/* pack here so that the box gets inserted after the recurrence rule frame */
	gtk_box_pack_start (GTK_BOX (ee->recur_vbox), ee->recur_hbox, FALSE, FALSE, 0);

	ee_rp_init_ending_date (ee);
	ee_rp_init_exceptions (ee, GTK_BOX (ee->recur_hbox));

	return ee->recur_vbox;
}

static void
event_editor_init_widgets (EventEditor *ee)
{	
	GtkWidget *notebook;
	GtkWidget *page;

	/* Buttons */

	/* FIXME: ok/cancel -> close */
        gtk_dialog_add_buttons (GTK_DIALOG (ee),
				GTK_STOCK_HELP,   GTK_RESPONSE_HELP,
				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_OK,     GTK_RESPONSE_OK,
				NULL);

	gtk_dialog_set_default_response (GTK_DIALOG (ee), GTK_RESPONSE_CLOSE);

	g_signal_connect (G_OBJECT (ee), "response",
			  G_CALLBACK (event_editor_response), NULL);

	/* Notebook */

	notebook = gtk_notebook_new ();
	gtk_widget_show (notebook);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(ee)->vbox), notebook,
			    TRUE, TRUE, 0);

	page = event_editor_create_general_page (ee);
	gtk_container_set_border_width (GTK_CONTAINER (page), GNOME_PAD_SMALL);
	gtk_widget_show_all (page);
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page,
				  gtk_label_new_with_mnemonic (_("General")));

	page = event_editor_create_recurrence_page (ee);
	gtk_container_set_border_width (GTK_CONTAINER (page), GNOME_PAD_SMALL);
	gtk_widget_show_all (page);
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page,
				  gtk_label_new_with_mnemonic (_("Recurrence")));
}

static void
event_editor_setup_gconf (EventEditor *ee)
{
	GConfClient *gconf;

	gconf = gconf_client_get_default ();

	gconf_client_add_dir (gconf, CONFIG_GENERAL_DIR,
			      GCONF_CLIENT_PRELOAD_NONE, NULL);
	gconf_client_add_dir (gconf, CONFIG_CALENDAR_DIR,
			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);

	/* Read configuration */

	ee->priv->week_starts_sunday =
		gconf_client_get_bool (gconf, CONFIG_GENERAL_WEEK_START, NULL);
	ee->priv->am_pm_flag =
		gconf_client_get_bool (gconf, CONFIG_GENERAL_AM_PM, NULL);
	ee->priv->day_begin =
		gconf_client_get_int (gconf, CONFIG_CAL_DAY_START, NULL);
	ee->priv->day_end =
		gconf_client_get_int (gconf, CONFIG_CAL_DAY_END, NULL);

	/* Notifications */

	ee->priv->notify1 =
		gconf_client_notify_add (gconf, CONFIG_GENERAL_WEEK_START,
					 event_edit_general_config_changed, ee,
					 NULL, NULL);
	ee->priv->notify2 =
		gconf_client_notify_add (gconf, CONFIG_GENERAL_AM_PM,
					 event_edit_general_config_changed, ee,
					 NULL, NULL);
	ee->priv->notify3 =
		gconf_client_notify_add (gconf, CONFIG_CAL_DAY_START,
					 event_edit_calendar_config_changed, ee,
					 NULL, NULL);
	ee->priv->notify4 =
		gconf_client_notify_add (gconf, CONFIG_CAL_DAY_END,
					 event_edit_calendar_config_changed, ee,
					 NULL, NULL);

	g_object_unref (G_OBJECT (gconf));
}

static void
event_editor_init (EventEditor *ee)
{
	int i;

	ee->priv = g_new0 (EventEditorPriv, 1);

	ee->priv->week_starts_sunday	= DEFAULT_WEEK_START;
	ee->priv->am_pm_flag		= DEFAULT_AM_PM;
	ee->priv->day_begin		= DEFAULT_CAL_DAY_START;
	ee->priv->day_end		= DEFAULT_CAL_DAY_END;

	ee->general              = NULL;
	ee->general_time_table   = NULL;
	ee->general_allday       = NULL;
	ee->general_owner        = NULL;
	ee->general_summary      = NULL;
	ee->general_radios       = NULL;

	ee->recur_page_label     = NULL;
	ee->recur_vbox           = NULL;
	ee->recur_hbox           = NULL;

	ee->recur_rr_group       = NULL;
	ee->recur_rr_notebook    = NULL;
	ee->recur_rr_day_period  = NULL;
	ee->recur_rr_week_period = NULL;

	for (i = 0; i < 7; i++)
		ee->recur_rr_week_days [i] = NULL;

	ee->recur_rr_month_date       = NULL;
	ee->recur_rr_month_date_label = NULL;
	ee->recur_rr_month_day        = NULL;
	ee->recur_rr_month_weekday    = NULL;
	ee->recur_rr_month_period     = NULL;
	ee->recur_rr_year_period      = NULL;

	ee->recur_ed_group            = NULL;
	ee->recur_ed_end_after        = NULL;

	ee->recur_ex_vbox             = NULL;
	ee->recur_ex_list             = NULL;
	
        ee->gnome_cal                 = NULL;
}

static void
event_editor_dispose (GObject *object)
{
	EventEditorPriv *priv;
	GConfClient *gconf;

	g_return_if_fail (object != NULL);
	g_return_if_fail (IS_EVENT_EDITOR (object));

	priv = EVENT_EDITOR (object)->priv;

	if (priv->ical)
		priv->ical->user_data = NULL; /* we are no longer editing it */

	gconf = gconf_client_get_default ();

	if (priv->notify1) {
		gconf_client_notify_remove (gconf, priv->notify1);
		priv->notify1 = 0;
	}
	if (priv->notify2) {
		gconf_client_notify_remove (gconf, priv->notify2);
		priv->notify2 = 0;
	}
	if (priv->notify3) {
		gconf_client_notify_remove (gconf, priv->notify3);
		priv->notify3 = 0;
	}
	if (priv->notify4) {
		gconf_client_notify_remove (gconf, priv->notify4);
		priv->notify4 = 0;
	}

	g_object_unref (G_OBJECT (gconf));

	G_OBJECT_CLASS (parent_class)->dispose (object);
}

static void
event_editor_finalize (GObject *object)
{
	EventEditor *ee;

	g_return_if_fail (object != NULL);
	g_return_if_fail (IS_EVENT_EDITOR (object));

	ee = EVENT_EDITOR (object);

	g_free (ee->priv);
	ee->priv = NULL;

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

GtkWidget *
event_editor_new (GnomeCalendar *owner, iCalObject *ical)
{
	EventEditor *ee;

	ee = g_object_new (TYPE_EVENT_EDITOR, NULL);

	if (!ical) {
		ical = ical_new_with_defaults ();
		ical->new = 1;
	} 

	if (ical->new)
		gtk_window_set_title (GTK_WINDOW (ee), _("Create new appointment"));
	else
		gtk_window_set_title (GTK_WINDOW (ee), _("Edit appointment"));

	ical->user_data = ee; /* so that the world can know we are editing it */

	ee->priv->ical = ical;
	ee->gnome_cal  = owner;

	event_editor_setup_gconf (ee);

	event_editor_init_widgets (ee);
	
	return GTK_WIDGET (ee);
}

/* FIXME: ugly, but currently necessary */
GtkWidget *
event_editor_new_whole_day (GnomeCalendar *owner, time_t day)
{
	EventEditor *ee;
	struct tm tm;
	iCalObject *ico;

	g_return_val_if_fail (owner != NULL, NULL);
	g_return_val_if_fail (GNOME_IS_CALENDAR (owner), NULL);

	ee = g_object_new (TYPE_EVENT_EDITOR, NULL);

	ee->gnome_cal = owner;

	event_editor_setup_gconf (ee);

        ico = ical_new_with_defaults ();
        ico->new = TRUE;

	ee->priv->ical = ico;

	event_editor_init_widgets (ee);

	tm = *localtime (&day);

	/* Set the start time of the event to the beginning of the day */

	tm.tm_hour = ee->priv->day_begin;
	tm.tm_min  = 0;
	tm.tm_sec  = 0;
	ico->dtstart = mktime (&tm);

	/* Set the end time of the event to the end of the day */

	tm.tm_hour = ee->priv->day_end;
	ico->dtend = mktime (&tm);

	return GTK_WIDGET (ee);
}
