/* GnomeCard - a graphical contact manager.
 *
 * card.h: This file is part of GnomeCard.
 * $Id: card.h,v 1.21.4.5 2002/07/06 12:22:07 srittau Exp $
 * 
 * Copyright (C) 1999 The Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Changed in 2002 by Sebastian Rittau <srittau@jroger.in-berlin.de>
 * for the GNOME 2 port.
 */

#ifndef CARD_H
#define CARD_H

#include <stdio.h>
#include <time.h>

#include <glib.h>

#include "cardtypes.h"

#define PO 0
#define EXT 1
#define STREET 2
#define CITY 3
#define REGION 4
#define CODE 5
#define COUNTRY 6

#define DELADDR_MAX 7

typedef struct _group
{
	gchar *name;
	struct _group *parent;
} CardGroup;

typedef struct
{
	gchar *name;
	gchar *data;
} CardXAttribute;

typedef struct
{
	CardGroup *grp;

	gint used;
	enum PropertyType type;
	enum EncodType encod;
	enum ValueType value;
	gchar *charset;
	gchar *lang;
	GList *xtension;
	
	void *user_data;
} CardProperty;

typedef struct
{
	CardProperty prop;
	
	gchar *str;
} CardStrProperty;

typedef struct
{
	CardProperty prop;
	
	gchar *name;
	gchar *data;
} CardXProperty;

typedef struct
{
	CardProperty prop;
	
	GList *l;
} CardList;


/* IDENTIFICATION PROPERTIES */


typedef struct
{
	CardProperty prop;
	
	gchar *family;        /* Public */
	gchar *given;         /* John */
	gchar *additional;    /* Quinlan */
	gchar *prefix;        /* Mr. */
	gchar *suffix;        /* Esq. */
} CardName;

typedef struct 
{
	CardProperty prop;

	enum PhotoType type;
	guint size;
	gchar *data;
} CardPhoto;

typedef struct
{
	CardProperty prop;
	
	gint year;
	gint month;
	gint day;
} CardBDay;


/* DELIVERY ADDRESSING PROPERTIES */

typedef struct
{
	CardProperty prop;
	
	gint type;
	gchar *data[DELADDR_MAX];
} CardDelAddr;            /* Delivery Address */

typedef struct 
{
	CardProperty prop;
	
	gint type;
	gchar *data;
} CardDelLabel;

/* TELECOMMUNICATIONS ADDRESSING PROPERTIES */


typedef struct
{
	CardProperty prop;
	
	gint type;
	gchar *data;
} CardPhone;

typedef struct 
{
	CardProperty prop;
	
	enum EMailType type;
	gchar *data;
} CardEMail;

typedef struct
{
	CardProperty prop;
	
	gint sign;      /* 1 or -1 */
	gint hours;     /* Mexico General is at -6:00 UTC */
	gint mins;      /* sign -1, hours 6, mins 0 */
} CardTimeZone;

typedef struct
{
	CardProperty prop;
	
	gfloat lon;
	gfloat lat;
} CardGeoPos;


/* ORGANIZATIONAL PROPERTIES */


typedef struct
{
	CardProperty prop;
	
	gchar *name;
	gchar *unit1;
	gchar *unit2;
	gchar *unit3;
	gchar *unit4;
} CardOrg;


/* EXPLANATORY PROPERTIES */


typedef struct
{
	CardProperty prop;
	
	gint utc;
	struct tm tm;
} CardRev;

typedef struct
{
	CardProperty prop;
	
	enum SoundType type;
	guint size;
	gchar *data;
} CardSound;

typedef struct
{
	CardProperty prop;
	
	enum KeyType type;
	gchar *data;
} CardKey;

typedef struct _Card
{
	CardProperty prop;
	
	CardStrProperty fname;
	CardName        name;
	CardPhoto       photo;
	CardBDay        bday;

        CardList        deladdr;
	CardList        dellabel;
	CardList        phone;
	CardList        email;
	CardStrProperty mailer;
	
	CardTimeZone    timezn;
	CardGeoPos      geopos;
	
	CardStrProperty title;
	CardStrProperty role;
	CardPhoto       logo;
	struct _Card   *agent;
	CardOrg         org;

	CardStrProperty categories;	
	CardStrProperty comment;
	CardRev         rev;
	CardSound       sound;
	CardStrProperty url;
	CardStrProperty uid;
	CardKey         key;
	
	CardList        xtension;
	
	gboolean        edited; /* is the card being edited? */
} Card;

extern Card         *card_new (void);
extern void          card_free (Card *crd);
extern void          card_prop_free (CardProperty prop);
extern CardProperty  card_prop_empty (void);
extern gint          card_check_prop (CardProperty prop);
extern gchar        *card_to_vobj_string (Card *card);
extern gchar        *card_to_string (Card *card);

extern gchar *card_bday_str (CardBDay bday);
extern gchar *card_rev_str(CardRev rev);
extern gchar *card_timezn_str (CardTimeZone timezn);
extern gchar *card_geopos_str (CardGeoPos geopos);

#endif
